package nl.iobyte.themeparkconnector;

import com.google.gson.Gson;
import net.milkbowl.vault.economy.Economy;
import nl.iobyte.themeparkconnector.api.ThemeParkConnectorAPI;
import nl.iobyte.themeparkconnector.api.config.enums.StorageKey;
import nl.iobyte.themeparkconnector.commands.ThemeParkConnectorCommand;
import nl.iobyte.themeparkconnector.listeners.*;
import nl.iobyte.themeparkconnector.sbd.License;
import nl.iobyte.themeparkconnector.sbd.UpdateManager;
import org.bukkit.Bukkit;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.RegisteredServiceProvider;
import org.bukkit.plugin.java.JavaPlugin;

public class ThemeParkConnector extends JavaPlugin {

    private final static Gson gson = ThemeParkConnectorGson.getInstance();

    private static ThemeParkConnector instance;
    private final ThemeParkConnectorAPI api = new ThemeParkConnectorAPI();
    private Economy econ = null;
    private boolean disabling = false;

    public void onEnable() {
        if (!setupEconomy() ) {
            Bukkit.getLogger().info(String.format("[%s] - No Vault dependency found!", getDescription().getName()));
        }

        instance = this;
        api.enable();

        //License stuff
        new License(this, "TPP", api.getConfigurationManager().getString(StorageKey.LICENSE));

        loadCommands();
        loadListeners();

        //Check Version
        new UpdateManager(this, 4, UpdateManager.CheckType.SBDPLUGINS).handleResponse((versionResponse, version) -> {
            if (versionResponse == UpdateManager.VersionResponse.FOUND_NEW) {
                Bukkit.getLogger().warning("[ThemeParkConnector] There is a new version available! Curent: " + getDescription().getVersion() + " New: " + version);
            } else if (versionResponse == UpdateManager.VersionResponse.LATEST) {
                Bukkit.getLogger().info("[ThemeParkConnector] You are running the latest version [" + getDescription().getVersion() + "]!");
            } else if (versionResponse == UpdateManager.VersionResponse.UNAVAILABLE) {
                Bukkit.getLogger().severe("[ThemeParkConnector] Unable to perform an update check.");
            }
        }).check();
    }

    private boolean setupEconomy() {
        if(getServer().getPluginManager().getPlugin("Vault") == null)
            return false;

        RegisteredServiceProvider<Economy> rsp = getServer().getServicesManager().getRegistration(Economy.class);
        if (rsp == null)
            return false;

        econ = rsp.getProvider();
        return econ != null;
    }

    private void loadCommands() {
        new ThemeParkConnectorCommand();
    }

    private void loadListeners() {
        PluginManager pm = Bukkit.getPluginManager();
        pm.registerEvents(new PlayerListener(), this);
        pm.registerEvents(new OperatorListener(), this);
        pm.registerEvents(new AttractionListener(), this);
        pm.registerEvents(new RideCountListener(), this);
        pm.registerEvents(new TicketListener(), this);
        pm.registerEvents(new RideOperatorListener(), this);
    }

    public void onDisable() {
        disabling = true;
        api.disable();

        instance = null;
    }

    public static Gson getGson() {
        return gson;
    }

    public static ThemeParkConnector getInstance() {
        return instance;
    }

    public Economy getEconomy() {
        return econ;
    }

    public ThemeParkConnectorAPI getAPI() {
        return api;
    }

    public boolean isDisabling() {
        return disabling;
    }
}
