package nl.iobyte.themeparkconnector.api.authentication;

import de.mkammerer.snowflakeid.SnowflakeIdGenerator;
import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.authentication.objects.WeakConcurrentHashMap;
import java.util.Map;
import java.util.UUID;

public class AuthenticationService {

    private final SnowflakeIdGenerator generator = SnowflakeIdGenerator.createDefault(42);
    private final Map<UUID, String> tokens = new WeakConcurrentHashMap<>(ThemeParkConnector.getInstance(), 15 * 1000);
    private final Map<UUID, String> attractions = new WeakConcurrentHashMap<>(ThemeParkConnector.getInstance(), 15 * 1000);

    public String generateToken(UUID uuid, String id) {
        long snowflake = generator.next();

        //ID to Token and store Token
        String token = Long.toString(snowflake);
        tokens.put(uuid, token);
        attractions.put(uuid, id);

        return token;
    }

    public boolean hasToken(UUID uuid) {
        return tokens.containsKey(uuid);
    }

    public boolean matchToken(UUID uuid, String token, String id) {
        String str = tokens.get(uuid);
        if(str == null || str.isEmpty())
            return false;

        if(!str.equals(token))
            return false;

        str = attractions.get(uuid);
        if(str == null || str.isEmpty())
            return false;

        if(!str.equals(id))
            return false;

        tokens.remove(uuid);
        attractions.remove(uuid);
        return true;
    }

}
