package nl.iobyte.themeparkconnector.api.client;

import net.md_5.bungee.api.chat.ClickEvent;
import net.md_5.bungee.api.chat.HoverEvent;
import net.md_5.bungee.api.chat.TextComponent;
import nl.iobyte.themepark.scheduler.ThemeParkScheduler;
import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.config.enums.StorageKey;
import nl.iobyte.themeparkconnector.api.message.MessageKey;
import nl.iobyte.themeparkconnector.api.state.objects.AbstractState;
import nl.iobyte.themeparkconnector.api.state.states.IdleState;

public class Publisher {

    private final ClientConnection clientConnection;

    public Publisher(ClientConnection clientConnection) {
        this.clientConnection = clientConnection;
    }

    /**
     * Attempt to generate url
     */
    public void startClientSession(String id) {
        if(id == null)
            return;

        //Check if already connected
        if (clientConnection.isConnected()) {
            MessageKey.CLIENT_CONNECTION_EXISTS.send(clientConnection.getPlayer());
            return;
        }

        if(!clientConnection.isWaitingForToken())
            MessageKey.CLIENT_GENERATE_TO_CONNECT.send(clientConnection.getPlayer());

        //Check if state allows connections
        AbstractState currentState = ThemeParkConnector.getInstance().getAPI().getStateService().getCurrentState();
        if(!currentState.canClientConnect()) {
            clientConnection.setWaitingForToken(true, id);
            if(currentState instanceof IdleState)
                ThemeParkScheduler.runAsync(() -> ThemeParkConnector.getInstance().getAPI().getNetworkingService().connectIfDown());

            return;
        }

        //Sending waiting message
        clientConnection.setWaitingForToken(true, id);
        String token = ThemeParkConnector.getInstance().getAPI().getAuthenticationService().generateToken(clientConnection.getOwnerUUID(), id);

        String url = ThemeParkConnector.getInstance().getAPI().getConfigurationManager().getString(StorageKey.SOCKET_PANEL_URL);
        url = url.replaceAll("%TOKEN%", token);
        url = url.replaceAll("%ID%", id);

        TextComponent message = new TextComponent(
                MessageKey.CLIENT_CLICK_TO_CONNECT.getMessage()
        );

        TextComponent[] hover = new TextComponent[]{
                new TextComponent(
                        MessageKey.CLIENT_HOVER_TO_CONNECT.getMessage()
                )
        };

        message.setClickEvent(new ClickEvent(ClickEvent.Action.OPEN_URL, url));
        message.setHoverEvent(new HoverEvent(HoverEvent.Action.SHOW_TEXT, hover));

        clientConnection.getPlayer().spigot().sendMessage(message);
        clientConnection.setWaitingForToken(false, null);
    }

}