package nl.iobyte.themeparkconnector.listeners;

import com.google.gson.Gson;
import de.tr7zw.changeme.nbtapi.NBTCompound;
import de.tr7zw.changeme.nbtapi.NBTItem;
import nl.iobyte.themepark.utils.LocationUtil;
import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.config.enums.StorageKey;
import nl.iobyte.themeparkconnector.api.event.ticket.TicketScanEvent;
import nl.iobyte.themeparkconnector.api.message.Text;
import nl.iobyte.themeparkconnector.api.show.objects.Ticket;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.block.SignChangeEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.inventory.ItemStack;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class TicketListener implements Listener {

    private final SimpleDateFormat format = new SimpleDateFormat("dd-MM-yyyy");

    @EventHandler
    public void onPlace(SignChangeEvent e) {
        if(e.getLines().length < 4)
            return;

        if(!e.getLine(0).equalsIgnoreCase(ThemeParkConnector.getInstance().getAPI().getConfigurationManager().getString(StorageKey.SIGN_SCAN_NAME)))
            return;

        Player player = e.getPlayer();
        if(e.getLine(1) == null || e.getLine(1).isEmpty()) {
            player.sendMessage(Text.color("&4Can't make a TicketScanner with empty show ID."));
            return;
        }

        if(e.getLine(2) == null || e.getLine(2).isEmpty()) {
            player.sendMessage(Text.color("&4Can't make a TicketScanner with empty show Date."));
            return;
        }

        String date = e.getLine(2);
        if(isValidFormat("HH:mm", date) && isValidFormat("HH:mm dd-MM-yyyy", date)) {
            player.sendMessage(Text.color("&4Can't make a TicketScanner with incorrect date, example: 20:00 or 20:00 04-04-2019"));
            return;
        }

        if(e.getLine(3) == null || e.getLine(3).isEmpty()) {
            player.sendMessage(Text.color("&4Can't make a TicketScanner with empty teleport location."));
            return;
        }

        String world = player.getLocation().getWorld().getName();
        if(LocationUtil.fromString(world + ":" + e.getLine(3)) == null) {
            player.sendMessage(Text.color("&4Can't make a TicketScanner with incorrect teleport location."));
            return;
        }

        e.setLine(0, Text.color(ThemeParkConnector.getInstance().getAPI().getConfigurationManager().getString(StorageKey.SIGN_SCAN_TITLE)));
        player.sendMessage(Text.color("&aSuccessfully made a TicketScanner"));
    }

    @EventHandler
    public void onScan(PlayerInteractEvent e) {
        if(e.getAction() == Action.LEFT_CLICK_AIR || e.getAction() == Action.RIGHT_CLICK_AIR)
            return;

        if(e.getClickedBlock() == null)
            return;

        Block block = e.getClickedBlock();
        if(!(block.getState() instanceof Sign))
            return;

        Sign sign = (Sign) block.getState();
        if(!sign.getLine(0).equals(Text.color(ThemeParkConnector.getInstance().getAPI().getConfigurationManager().getString(StorageKey.SIGN_SCAN_TITLE)))) {
            return;
        }

        String id = ChatColor.stripColor(sign.getLine(1));
        if(id == null || id.isEmpty())
            return;

        String date = ChatColor.stripColor(sign.getLine(2));
        if(date == null || date.isEmpty())
            return;

        if(isValidFormat("HH:mm", date) && isValidFormat("HH:mm dd-MM-yyyy", date)) {
            System.out.println("Invalid date format");
            return;
        }

        if(isValidFormat("HH:mm dd-MM-yyyy", date))
            date = date + " " + format.format(new Date());

        Location location = LocationUtil.fromString(sign.getLocation().getWorld().getName() + ":" + sign.getLine(3));
        if(location == null) {
            System.out.println("Invalid Location");
            return;
        }

        if(e.getItem() == null)
            return;

        ItemStack item = e.getItem();
        if(item.getType() != Material.PAPER)
            return;

        NBTItem it = new NBTItem(item);
        if(!it.hasKey("data")) {
            System.out.println("Missing data key");
            return;
        }

        NBTCompound compound = it.getCompound("data");
        if(!compound.hasKey("ticket")) {
            System.out.println("Missing ticket key");
            return;
        }

        Gson g = new Gson();
        Ticket ticket = g.fromJson(it.getCompound("data").getString("ticket"), Ticket.class);
        if(ticket == null) {
            System.out.println("Couldn't get ticket class using Gson");
            return;
        }

        if(!id.equals(ticket.getShowID())) {
            System.out.println("Different show id");
            return;
        }

        if(!date.equals(new SimpleDateFormat("HH:mm dd-MM-yyyy").format(ticket.getShowDate()))) {
            System.out.println("Different date");
            return;
        }

        Player player = e.getPlayer();
        boolean b = ThemeParkConnector.getInstance().getAPI().getShowService().setValid(ticket);

        TicketScanEvent event = new TicketScanEvent(player.getUniqueId(), ticket, b);
        Bukkit.getPluginManager().callEvent(event);

        int i = player.getInventory().getHeldItemSlot();
        player.getInventory().remove(player.getInventory().getItem(i));
        if(!b) {
            player.sendMessage(Text.color("&4Invalid ticket."));
            return;
        }

        if(!event.isCancelled())
            player.teleport(location);

        player.sendMessage(Text.color("&6Used ticket for Show: " + item.getItemMeta().getDisplayName()));
    }

    private static boolean isValidFormat(String format, String value) {
        try {
            SimpleDateFormat sdf = new SimpleDateFormat(format);
            Date date = sdf.parse(value);
            if(!value.equals(sdf.format(date)))
                date = null;

            return date == null;
        } catch(ParseException ex) {
            return true;
        }
    }

}
